package egotham.flexibleloadcontrol;

import egotham.communication.Connection;
import egotham.configuration.Configuration;
import egotham.sdi.Clock;
import egotham.sdi.endpoints.Source;
import egotham.sdi.history.HistoryClient;
import egotham.sdi.history.SeriesQuery;
import egotham.sdi.model.Accumulator;
import egotham.sdi.model.Analog;
import egotham.sdi.model.Command;
import egotham.sdi.model.Discrete;
import egotham.sdi.model.DiscreteValue;
import egotham.sdi.model.RealValue;
import egotham.sdi.model.StringMeasurement;
import egotham.sdi.model.StringValue;
import egotham.sdi.ontology.OntologyClient;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 * Implementation of InOutAll with connections to the API
 *
 * @author thaddauh
 */
public class InOutAllAPIConnect implements InOutAll {

    private Settings settings;
    private Configuration config;
    private Connection conHandle;
    private HistoryClient historyHandle;
    private OntologyClient ontHandle;
    private String egboxHandle;

    public InOutAllAPIConnect() {
        try {
            this.config = Configuration.Defaults();
            this.conHandle = Connection.Create(this.config, true);
            this.historyHandle = new HistoryClient(this.conHandle, this.config);
            this.ontHandle = new OntologyClient(this.conHandle, this.config);
            this.egboxHandle = this.config.getString("sdi.domain", "");

        } catch (Exception e) {
            Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, e);
        }

    }

    /**
     * returns the current e-gotham timestamp in the system.
     *
     * @return
     */
    @Override
    public long getCurrentTime() {
        return Clock.getInstance().currentTimestamp();

    }

    public Settings getSettings() {
        return settings;
    }

    public void setSettings(Settings settings) {
        this.settings = settings;
    }

    /**
     * Returns a request for consumption Data as a List of RealValues If
     * startTime is earlier than the startUpTime of the system, startTime is
     * replaced by startUpTime
     *
     * @param relayNumber
     * @param startTime Time in egotham time (converted Java time in clock()
     * @param endTime Time in egotham time (converted Java time in clock()
     * @param resultLimit 0, all values are returned 0>N returns N last Samples
     * 0<N returns N first Samples 
     * @return List<RealValue> Java List with RealValues
     */
    @Override
    public List<RealValue> consumptionData(int relayNumber, long startTime, long endTime, int resultLimit) {
        List<RealValue> result = new ArrayList<>();
        if (startTime < settings.getStartUpTime())//not enough data is existing
        {
            if (endTime < settings.getStartUpTime()) {//interval is entirely before startTime.
                startTime = 0;
                endTime = settings.getStartUpTime();
            } else {
                startTime = settings.getStartUpTime();
            }
        }
        try {
            Analog consumption = ontHandle.find(Analog.class, this.egboxHandle + ".relay." + String.valueOf(relayNumber) + ".power");

            SeriesQuery query = historyHandle.createSeriesQuery(consumption);

            query.setStartTime(startTime);
            query.setEndTime(endTime);
            query.setResultLimit(resultLimit);

            result = query.execute();

        } catch (Exception ex) {
            Logger.getLogger(InOutAll.class.getName()).log(Level.SEVERE, null, ex);
        }

        return result;
    }

    /**
     * Activates the relay with number relayNumber
     *  does not check if relay exists
     *
     * @param relayNumber
     * @return true if action was performed
     * @throws java.lang.Exception
     */

    @Override
    public boolean activateRelay(int relayNumber) throws Exception {

        if (!statusRelay(relayNumber)) {
            Command relay = this.ontHandle.find(Command.class, this.egboxHandle + ".relay." + String.valueOf(relayNumber) + ".command");
            Source relay1Source = new Source(conHandle, relay);
            relay1Source.send(new DiscreteValue(1));
            return true;
        }
        return false;
    }

    /**
     * Deactivates the relay with number relayNumber
     *  does not check if relay exists
     *
     * @param relayNumber
     * @return true if deactivation was performed
     * @throws java.lang.Exception
     */
    @Override
    public boolean deactivateRelay(int relayNumber) throws Exception {

        if (statusRelay(relayNumber)) {
            Command relay = this.ontHandle.find(Command.class, this.egboxHandle + ".relay." + String.valueOf(relayNumber) + ".command");
            Source relay1Source = new Source(conHandle, relay);
            relay1Source.send(new DiscreteValue(0));
            return true;
        }
        return false;

    }

    /**
     * Checks the status of the relay (on/off) and returns it
     *
     * @param relayNumber
     * @return status of the relay(true on, false off))
     * @throws java.lang.Exception
     */
    @Override
    public boolean statusRelay(int relayNumber) throws Exception {

        Discrete relay = this.ontHandle.find(Discrete.class, this.egboxHandle + ".relay." + String.valueOf(relayNumber) + ".state");
        SeriesQuery query = this.historyHandle.createSeriesQuery(relay);
        query.setEndTime(getCurrentTime());
        query.setResultLimit(-1);
        List<DiscreteValue> execute = query.execute();
        return (execute.get(0).v == 1);
    }

    /**
     * returns the Energy consumption of the total system in W*h
     *
     * @param N 0, all values are returned,
     * 0>N returns N last Samples
     * 0<N returns N first Samples @param startTime
     * egotham timestamp for the start of the interval to be returned
     * @param startTime egotham timestamp for the beginning
     * @param endTime egotham timestamp for the end of the interval returned
     * @return List<RealValue> with results for the interval
     * @throws Exception
     */
    @Override
    public List<DiscreteValue> fullConsumption(int N, long startTime, long endTime) throws Exception {
        if (startTime < settings.getStartUpTime())//not enough data is existing
        {
            if (endTime < settings.getStartUpTime()) {//interval is entirely before startTime.
                startTime = 0;
                endTime = settings.getStartUpTime();
            } else {
                startTime = settings.getStartUpTime();
            }
        }

        Accumulator consumption = ontHandle.find(Accumulator.class, egboxHandle + ".ams.energy");

        SeriesQuery query = historyHandle.createSeriesQuery(consumption);
        query.setEndTime(endTime);
        query.setStartTime(startTime);
        query.setResultLimit(N);

        return query.execute();
    }

    /**
     * Reads the Tariff from the database closest to the end time and returns it
     * as a long[24][3] array if the tariff at the requested timestamp is after
     * the starting time the returned tariff is the first received after
     *
     * @param endTime egotham timestamp directly after the retrieved Tariff
     * @return type long[24][3], 24 sets of timestamp[0], low price [1] and high
     * price [2]
     * @throws Exception database call
     */
    @Override
    public long[][] gettariff(long endTime) throws Exception {

        int N = -1;

        if (settings.getStartUpTime() >= endTime) {
            endTime = settings.getStartUpTime();
 
        }


        StringMeasurement jsonTariff = ontHandle.find(StringMeasurement.class, egboxHandle + ".tariff");
        SeriesQuery query = historyHandle.createSeriesQuery(jsonTariff);
        query.setEndTime(endTime);
        query.setResultLimit(N);   // gets the latest tariff        
        List<StringValue> result = query.execute();
        String s = result.get(0).v;

 return readJsonTariff(s);// takes the Json String apart and returns it as long[24][3]


  

    }

    /**
     * Takes a Json string for the tariff and returns it as long array
     *
     * @param message Json string with tariff for 24h low prices and high prices
     * @return long[24][3],24 units of timestamp[0], low price[1], high price[2]
     * @throws ParseException
     */
    private long[][] readJsonTariff(String message) throws ParseException {
        long[][] returnValue = new long[24][3];

        JSONParser parser = new JSONParser();
        JSONObject obj = (JSONObject) parser.parse(message);
        JSONObject tmpObj;
        String tmp = (String) obj.get("firstHour");

        long firstHour = Long.parseLong(tmp);//starting point
        long currentTime;
        for (int i = 0; i < 24; i++) {

            currentTime = firstHour + i * 3600000;// one hour iteration
            returnValue[i][0] = currentTime;
            tmpObj = (JSONObject) obj.get(String.valueOf(currentTime)); // get the next Json object  
            returnValue[i][1] = (long) tmpObj.get("lowPrice");        // low price of the timestamp
            returnValue[i][2] = (long) tmpObj.get("highPrice");           // high price of the timestamp
        }

        return returnValue;
    }
/**
 * Returns the latest temperature from concerning sensor
 * @param sensorNumber  number corresponding to database
 * @return returns double temperature in Celsius
 */
    @Override
    public double getTemperature(int sensorNumber) {

        Analog temperature = ontHandle.find(Analog.class, egboxHandle + ".sensor." + String.valueOf(sensorNumber) + ".temperature");
        SeriesQuery query = historyHandle.createSeriesQuery(temperature);
        List<RealValue> result = new ArrayList<>();
        try {
            query.setResultLimit(-1);
            result = query.execute();
        } catch (Exception ex) {
            Logger.getLogger(InOutAll.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (result.isEmpty()) {
            return 0;
        } else {
            return result.get(0).v;
        }

    }

    /**
     * Reads the Database and takes the consumption cap from the latest config
     * input
     *
     * @return Consumption cap in W*h
     */
    @Override
    public double getConsumptionCap() {
        List<StringValue> result;
        double returnValue = 0;
        try {
            StringMeasurement consumption = ontHandle.find(StringMeasurement.class, egboxHandle + ".config");

            SeriesQuery query = historyHandle.createSeriesQuery(consumption);
            query.setResultLimit(-1);   //latest input

            result = query.execute();
            JSONParser parser = new JSONParser();
            JSONObject obj = (JSONObject) parser.parse(result.get(0).v);
            long tmp = (long) obj.get("subscribedEnergy");
            returnValue = (double) tmp;

        } catch (Exception ex) {
            Logger.getLogger(InOutAll.class.getName()).log(Level.SEVERE, null, ex);
        }
        return returnValue;

    }

}
