package egotham.flexibleloadcontrol.SortingTechnique;

import egotham.flexibleloadcontrol.control.UtilityControl;
import egotham.flexibleloadcontrol.InOutAll;
import egotham.flexibleloadcontrol.Settings;

import egotham.sdi.model.DiscreteValue;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implements the sorting algorithm concerning the consumption cap
 *
 * @author ted
 */
public class CapSort implements SortingTechnique {

    private final InOutAll inoutall;
    private final Settings settings;
    private int iterCounter;
    private double startEnergy;

    public CapSort(InOutAll inoutall, Settings settings) {
        this.inoutall = inoutall;
        this.settings = settings;
        this.iterCounter = 0;
    }

    /**
     * Determines if the Price Cap is applicable and creates a schedule for
     * utilities for one interval. Priority 1 utilities are always scheduled
     *
     * @param utilities A list of utilities that need to be scheduled
     * @param totalUtilities A list of all utilities in the system.
     * @return A list of Utilities that is scheduled to follow the requirements.
     */
    @Override
    public List<UtilityControl> sort(List<UtilityControl> utilities, List<UtilityControl> totalUtilities) {
        List<UtilityControl> returnValue = new ArrayList<>();
        returnValue.addAll(utilities);
        int iterations = 60 / settings.getUpdateInterval();//iterations per hour
        try {
            if (useCap()) {   //if the cap is not in use there's no need to sort
                System.out.println("CapSort: Cap is applied");
                double cap = inoutall.getConsumptionCap();  //User specific consumption cap in W*h
                double staticEnergy = staticEnergyEstimation(totalUtilities);// needs utilities because only static+dynamic power is available, returns 
                double tmpEnergy;
                long currentTimestamp = inoutall.getCurrentTime();
                double availableEnergy;

                double currentEnergy = inoutall.fullConsumption(-1, 0, currentTimestamp).get(0).v;

                if (this.iterCounter == 0) {        // if this is the first iteration: start energy has to be set
                    startEnergy = currentEnergy;

                }

                // calculate available power for interval
                availableEnergy = cap - (currentEnergy - startEnergy); // the rest of the available energy for 1h
                availableEnergy = availableEnergy / (iterations - iterCounter); //divided by the amount of iterations left this hour
                availableEnergy = availableEnergy - staticEnergy / iterations; // subtract the available static energy available for one iteration

                // find which utilities can be active at this point
                for (UtilityControl util : utilities) {
                    tmpEnergy = util.getConsumptionEstimation() / iterations; // consumption estimation for the utility in W*h

                    if (!(util.getUrgency() == 1) && tmpEnergy > availableEnergy) {// if the utility doesn't have prio 1 or consumes more than the cap remove it 
                        returnValue.remove(util);
                    } else {
                        availableEnergy -= tmpEnergy;   //available energy is reduced by the running utility
                    }
                }

            } else {
                
            }
        } catch (Exception ex) {
            Logger.getLogger(CapSort.class.getName()).log(Level.SEVERE, null, ex);
        }

        if (iterCounter >= iterations) {    //If the iterations are at maximum
            iterCounter = 0;
        } else {
            iterCounter++;
        }
        return returnValue;
    }

    /**
     * takes the power consumption of the last hour and subtracts the estimated
     * consumption of all utilities
     *
     * @return the estimation for static power consumption in this hour in W*h
     */
    private double staticEnergyEstimation(List<UtilityControl> totalUtilities) throws Exception {
        long currentTime = inoutall.getCurrentTime();
        double totalPower;
        List<DiscreteValue> allPower = inoutall.fullConsumption(0, currentTime - 3600000L, currentTime);
        totalPower = allPower.get(allPower.size() - 1).v - allPower.get(0).v;//first value minus latest value to get static+dynamic consumption in last hour
        for (UtilityControl util : totalUtilities) {                //takes the 
            totalPower -= util.getConsumptionEstimation();//consumptionEstimation in W*h
        }
        return totalPower;
    }

    /**
     * Checks if the customer specific cap is acceptable to be used (the price
     * difference between high and low price is too high to be ignored)
     *
     * @return true if the price cap has to be used
     */
    private boolean useCap() {
        try {
            long[][] tariff = inoutall.gettariff(inoutall.getCurrentTime());

            if ((tariff[0][2] - tariff[0][1]) <= settings.getPriceDifference()) {
                return false;
            }

        } catch (Exception ex) {
            Logger.getLogger(CapSort.class.getName()).log(Level.SEVERE, null, ex);
        }
        return true;
    }
}
