package egotham.flexibleloadcontrol.control;

import egotham.flexibleloadcontrol.InOutAll;
import egotham.flexibleloadcontrol.Settings;
import egotham.sdi.model.RealValue;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implements the control for the Boiler
 *
 * settings is the main Settings class that updates periodically relaynumber is
 * the number the relay corresponds to in the database inoutall is the class
 * that regulates the access to the database and does the communication to the
 * "outside world"
 *
 * @author ted
 */
public class BoilerControl implements UtilityControl {

    private Settings settings; //is the main Settings class 
    private int relayNumber;
    private final InOutAll inoutall;
    private final long[] activityTimestamps;
    private long start;
    private long stop;
    private double avgConsumption;
    private boolean ongoingMeasurement;
    private double percentageOfUsage;

    public Settings getSettings() {
        return settings;
    }

    public void setSettings(Settings settings) {
        this.settings = settings;
    }

    public int getRelayNumber() {
        return relayNumber;
    }

    public void setRelayNumber(int relayNumber) {
        this.relayNumber = relayNumber;
    }

    /**
     * Constructor for boiler control
     *
     * @param settings settings class that updates automatically
     * @param relayNumber relay number as used in database
     * @param inoutall class that handles all communication to the outside world
     */
    public BoilerControl(int relayNumber, InOutAll inoutall, Settings settings) {
        this.settings = settings;
        this.relayNumber = relayNumber;
        this.inoutall = inoutall;
        this.avgConsumption = 0;
        this.percentageOfUsage = 0;
        this.activityTimestamps = new long[120 / settings.getUpdateInterval()];
        for (int i = 0; i < activityTimestamps.length; i++) {
            this.activityTimestamps[i] = 0;
        }
    }

    /**
     * Takes the data from the database from the start date to the finish, adds
     * them up and creates an average Easy way of calculating energy
     *
     * @return the average consumption of a relay between start and stop
     */
    private double iterateAvgConsumption() {
        List<RealValue> data = inoutall.consumptionData(relayNumber, start, stop, 0);
        double returnValue = 0;

        for (RealValue data1 : data) {
            returnValue += data1.v;
        }
        returnValue /= data.size();
        return returnValue;
    }

    /**
     * permits usage to the relay also used for measuring the average
     * consumption
     *
     * @param use represents if the relay is allowed to be active or not
     * @return true if operation successful, false if relay was already
     * active/inactive
     */
    @Override
    public boolean permitUsage(boolean use) {

        Boolean returnvalue = true;
        this.percentageOfUsage = iterateUsage();      //creting a percentage of usage for urgency
        try {
            if (use) {

                returnvalue = this.inoutall.activateRelay(relayNumber);      //returns if the activation was successful (switch from off->on)

                if (!ongoingMeasurement) {
                    ongoingMeasurement = true;
                    start = inoutall.getCurrentTime();
                }

            } else {

                returnvalue = this.inoutall.deactivateRelay(relayNumber);      //returns if the activation was successful (switch from on->off)  
                if (ongoingMeasurement) {
                    ongoingMeasurement = false;
                    stop = inoutall.getCurrentTime();
                    avgConsumption = iterateAvgConsumption();
                }

            }
        } catch (Exception ex) {
            Logger.getLogger(BoilerControl.class.getName()).log(Level.SEVERE, null, ex);
        }
        return returnvalue;

    }

    /**
     * Used to iterate the array to record usage and calculates current
     * percentage of use
     *
     * @return
     */
    private double iterateUsage() {
        long currentTime = inoutall.getCurrentTime();
        int count = 0;
        for (int i = 0; i < this.activityTimestamps.length - 1; i++) {
            this.activityTimestamps[0 + i] = this.activityTimestamps[1 + i];

        }//shifting the array once to the left
        try {
            if (this.inoutall.statusRelay(relayNumber)) {
                this.activityTimestamps[this.activityTimestamps.length - 1] = currentTime;
            } else {
                this.activityTimestamps[this.activityTimestamps.length - 1] = 0;
            }// adds a timestamp at the last space of the array, 0 if the relay is off
        } catch (Exception ex) {
            Logger.getLogger(BoilerControl.class.getName()).log(Level.SEVERE, null, ex);
        }

        for (int i = 0; i < this.activityTimestamps.length; i++) {
            if (this.activityTimestamps[i] == 0) {
                count++;
            }//counts the amount of times the relay was off

        }
        return (1 - (count / this.activityTimestamps.length));

    }

    /**
     * The Urgency reflects how important the controller evaluates its need of
     * usage time. The boiler controller checks the usage in the last two hours
     * and finds a corelation between activity and Priority
     *
     * @return urgency of the relay between 1-10 while 1 is high
     */
    @Override
    public int getUrgency() {

        if (this.percentageOfUsage > 0.5) {
            return 10;
        } else if (this.percentageOfUsage <= 0.5 && this.percentageOfUsage > 0.25) {
            return 5;
        } else if (this.percentageOfUsage <= 0.25 && this.percentageOfUsage > 0.1) {
            return 3;
        } else {
            return 1;
        }

    }

    /**
     *
     * @return returns the estimation of Energy consumption in W*h
     */
    @Override
    public double getConsumptionEstimation() {
        return avgConsumption * 3600; //Avg consumption for the next hour in W*h

    }

}
