package egotham.flexibleloadcontrol.control;

import egotham.flexibleloadcontrol.InOutAll;
import egotham.flexibleloadcontrol.Settings;

import egotham.sdi.model.RealValue;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implements the control for the Heater
 *
 * @author ted
 */
public class HeaterControl implements UtilityControl {

    private final Settings settings;//is the main Settings class that updates periodically
    private int relayNumber;// is the number the relay corresponds to in the database
    private final InOutAll inoutall;//is the class that regulates the access to the database and  does the communication to the "outside world"
    private int sensornumber;
    private long start;//starting timestamp for calculation of consumption
    private long stop;//endpoint timestamp for calculation of consumption
    private double avgConsumption;//average consumption of the last timestamp recorded
    private boolean ongoingMeasurement;// true if there is a measurement in progress

    public int getRelayNumber() {
        return relayNumber;
    }

    public void setRelayNumber(int relayNumber) {
        this.relayNumber = relayNumber;
    }

    public int getSensornumber() {
        return sensornumber;
    }

    public void setSensornumber(int sensornumber) {
        this.sensornumber = sensornumber;
    }

    /**
     *
     * @param relayNumber is the number the relay corresponds to in the database
     * @param sensornumber Number of the corresponding thermostat
     * @param inout is the number the relay corresponds to in the database
     * @param settings is the main Settings class that updates periodically
     */
    public HeaterControl(int relayNumber, int sensornumber, InOutAll inout, Settings settings) {
        this.relayNumber = relayNumber;
        this.inoutall = inout;
        this.settings = settings;
        this.avgConsumption = 0;
        this.sensornumber = sensornumber;
    }

    /**
     * permits usage to the relay also used for measuring the average
     * consumption
     *
     * @param use represents if the relay is allowed to be active or not
     * @return true if operation successful
     */
    @Override
    public boolean permitUsage(boolean use) {
        Boolean returnvalue = true;
        if (use) {
            try {
                returnvalue = this.inoutall.activateRelay(relayNumber);
                if (!ongoingMeasurement) {
                    ongoingMeasurement = true;
                    start = inoutall.getCurrentTime();
                }
            } catch (Exception ex) {
                Logger.getLogger(HeaterControl.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            try {
                returnvalue = this.inoutall.deactivateRelay(relayNumber);

                if (ongoingMeasurement) {
                    ongoingMeasurement = false;
                    stop = inoutall.getCurrentTime();
                    avgConsumption = (iterateAvgConsumption() + avgConsumption) / 2;    //average of the old value with the new
                }
            } catch (Exception ex) {
                Logger.getLogger(HeaterControl.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return returnvalue;
    }

    /**
     *
     * @return urgency of the relay between 1-10 while 1 is high
     */
    @Override
    public int getUrgency() {

        int bt = this.settings.getBestTemperature();
        int mint = this.settings.getLowTemperature();
        int maxt = this.settings.getHighTmeperature();

        double ct = this.inoutall.getTemperature(sensornumber);

        if (ct <= mint) {
            return 1;
        } else if (mint < ct && ct <= (bt - 1)) {
            return 2;
        } else if (bt - 1 < ct && ct <= bt + 1) {
            return 5;
        } else if (bt + 1 < ct && ct <= maxt) {
            return 6;
        } else if (maxt < ct) {
            return 10;
        } else {
            return 3;
        }

    }

    /**
     *
     * @return uses previous measurements and the newest to create an average
     * consumption
     */
    private double iterateAvgConsumption() {
        List<RealValue> data = inoutall.consumptionData(relayNumber, start, stop, 0);
        double returnValue = 0;

        for (RealValue data1 : data) {
            returnValue += data1.v;
        }
        returnValue /= data.size();
        return returnValue;
    }

    /**
     *
     * @return returns the estimation of Energy in W*h
     */
    @Override
    public double getConsumptionEstimation() {
        return avgConsumption * 3600;

    }

}
